# -*- coding: utf-8 -*-
"""
Created on Tue May 30 10:19:56 2023

@author: Imraan Alas
"""

#%% Section 0: Intentions for scripts:
    
# Section 0: Here
# Section 1: Information
# Section 2: Imports & Versions
# Section 3: Parameter Definitions
# Section 4: Data Processing (antiSMASH)
# Figure 1A: 
# Figure 1B:
# Section 5: Data Processing (BiG-SLICE & BiG-SCAPE)
# Figure 3: 
# Section 6: Intensive data-processing.
# Section 6.5: Once you've run Section 6 and saved the major output, you can start from here as long as you load the file appropriately.
# Figure 4:
# Figure 5:
# Section 7:
# Figure 6:
# Figure 7:
# Figure 8:
# Supplementary Figures: Group A
# Supplementary Figures: Group B
# Supplementary Figures: Group C
# Supplementary Figures: Group D
    
    
#%% Section 1: Information.

# Figures created using antiSMASH information.
    # Figure 1
# Figures created using BiG-SliCE information.
    # Figures 3, 4, 5, 6, 7, 8
# Figures created using BiG-SCAPE information with BiG-SliCE
    # Figures 4, 5, 6, 7, 8        

# Notes (disorganized):
    # The antiSMASH script only has been tested for antiSMASH version 5.1.1. You will need to modify the script to read antiSMASH version 6 or 7 outputs.
    # This was primarily done with the data I had. So, I end up pruning out a few genomes and BGCs based on quality control, lack of being a Micromonosporaceae, or BGCs found to belong to contamination.
    # This was pieced together from years of code, generated at disjointed times.
    
#%% Section 2: Imports & Versions

# Python version: 3.7.12, packaged by conda-forge (default, Oct 26 2021, 05:37:49). [MSC v.1916 64 bit (AMD64)]
# IPython 7.33.0

# ---- BiG-SCAPE & BiG-SLICE packages

import sqlite3 # sqlite version 3.38.5
import pandas as pd # pandas version 1.3.5
import numpy as np # numpy version 1.21.6
# import sqlalchemy
import matplotlib.pyplot as plt # matplotlib version 3.5.1
# from matplotlib import rcParams
import math
import seaborn as sns # seaborn version 0.11.2
# import networkx as nx
import sklearn # scikit-learn version 1.0.2
# from sklearn import metrics
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
# import graphviz
# import pygraphviz
from sklearn.manifold import TSNE
from sklearn.manifold import MDS
import scipy # scipy version 1.7.3
import plotly.express as px # plotly express version 0.4.1
import plotly.io as pio # plotly version 5.10.0
# from sklearn.cluster import KMeans, AffinityPropagation, AgglomerativeClustering, Birch, DBSCAN, MeanShift, OPTICS # k-means clustering
# from sklearn.manifold import TSNE # t-SNE clustering
# from sklearn.model_selection import train_test_split # training/testing splitter
from scipy.cluster.hierarchy import linkage, dendrogram # Hierarchical Clustering
from numpy import unique, where
# from decimal import Decimal
# from pandas import ExcelWriter
# from sklearn import tree
# from mpl_toolkits import mplot3d
from statannotations.Annotator import Annotator
import copy

# ----- antiSMASH packages 

# original packages:
import os
from bs4 import BeautifulSoup # beautiful soup version 4.10.0
from copy import deepcopy 
from os import chdir, getcwd
import re
# duplicates of above packages: Uncomment if you want to run only the antiSMASH scripts.
# import pandas as pd # pandas version 1.3.5
# import numpy as np # numpy version 1.21.6
# import matplotlib.pyplot as plt
# import seaborn as sns

#%% Section 3: Parameter Definitions

# Directory to save results to (csv, pdf, png, tiff, etc):
resultsDirectory = r"C:\Users\imraa\Documents\UWM\BugniLab\Genome Assembly\scriptOutputs";
print('Directory where results have been saved to: ' + resultsDirectory)

# ---- antiSMASH relevant parameters:
# The directory that the antiSMASH folders are in. This should be a folder that contains folders for each antiSMASH output. So, folder 1 will contain all the gbks, jsons, and outputs for one specific bacteria.
antiSMASHdirectory = r"C:\Users\imraa\Documents\UWM\BugniLab\Genome Assembly\micromonosporaceae-AntiSmash511-Clean";
print('AntiSMASH version 5.1.1 directory: ' + antiSMASHdirectory)

# The excel file that contains two columns.
    # Column 1: The antiSMASH Types predicted for every single region identified in antiSMASH. So, each entry may be anywhere from 1 to some number of antiSMASH predicted products for the one region.
        # Examples:
            # terpene
            # thiopeptide,LAP
            # siderophore, NRPS, T1PKS, PKS-like
    # Column 2: Using Big-SCAPE's classes for grouping together antiSMASH annotations into the categories of: PKS 1, PKS Other, NRPS, RiPPs, Saccharides, Terpene, PKS/NRPS Hybrids, Others
        # The rules can be seen here: https://github.com/medema-group/BiG-SCAPE/wiki/big-scape-classes#big-scape-classes
        # PKS/NRPS Hybrids were not annotated in here. Each individual antiSMASH annotation was assigned a BiG-SCAPE class, and written down.
        # This excel file was manually annotated. I scraped all of the "Type" column from the antiSMASH files, found only unique instances. Those unique instances formed column 1. I manually went through  and put into column 2 the BiG-SCAPE classes for that region.
        # Examples:
            # Column 1 (Types) | Column 2 (SuperClusters)
            # Lanthipeptide | RIPP
            # NAGGN | Other
            # T1PKS,NRPS | PKS/NRPS
            # siderophore, NRPS, T1PKS, PKS-like | PKS/NRPS/Siderophore
# This file can be partially generated by running Section 4, and uncommenting the line: fullTypeDFcounts.to_excel('allUniqueStringTypes.xlsx')
    # This will generate Column 1, you will have to manually annotate or write a script to fill in Column 2.
bigSCAPEclassesConversion = r'C:\Users\imraa\Documents\UWM\BugniLab\Yeast-Chemical-Genomics\renamedMicromonosporaTypesv42_antismash511_bigscapeRules.xlsx';
renamedAntiSMASHtypes_to_bigSCAPEclasses = pd.read_excel(bigSCAPEclassesConversion)
print('The file used to convert antiSMASH product types to BiG-SCAPE classes: ' + bigSCAPEclassesConversion)

# If you would like to re-order the output of Figure 1, here's where you can do it. This is from left to right. I matched it to the Phylogenetic Tree (top-down). It's used in Figure 1A: Plot Generation
# reOrderList = ['c500', 'd1127', 'd406', 'd791', 'a2056', 'a2059', 'd792', 'd937', 'd1102', 'd1047', 'a1363','d710','d1120','d961','d967','c250','d1155','d964','d882','d1082','c514','d573','d1129','d956','d712','d714','d987','a1998','d980','c241','d998','d1128','d975','d1076','d718','c273','a1976','b486','b482','c264','a1947','a1949'];
reOrderList = ['c500','d1127','d406','a2056','d791','a2059','d792','d937','d1047','d1102','d882','d1082','c514','d1129','d573','d710','d1120','d961','c250','d967','d964','d1155','d712','d956','d714','d987','a1998','d980','c241','d998','d975','d1128','c264','a1947','a1949','d1076','d718','c273','a1976'];
# If you would like to remove any strains, note them here.
strainsToRemove = ['a1363', 'b482', 'b486']
print('Strains to remove from the antiSMASH visualizations: ' + ','.join(strainsToRemove))
# ---- BiG-SLICE & BiG-SCAPE relevant parameters:

# Note: I ran BiG-SLiCE in Windows Subsystem for Linux, then copied the entire "full_run_result" folder over to Windows for ease of access, overwriting/merging the previous "full_run_result".

# This assumes you've already run the BiG-SliCE.
pathToDataDB = "C:/Users/imraa/Downloads/full_run_result/full_run_result/result/data.db"; # This is the path to the data.db file, once copied from Linux and put into Windows.
print('The path to the data.db file in the results folder: ' + pathToDataDB) 
pathToReportsDB = "C:/Users/imraa/Downloads/full_run_result/full_run_result/reports/";
print('The path to the reports folder: ' + pathToReportsDB)
run_id = 6; # This is the BiG-SLiCE data associated with the Threshold = 900. If you queried your BGCs against the 29,955 GCFs, runID 6 is what you want.
print('The run_id used for BiG-SLiCE (6 is associated with Threshold = 900): ' + str(run_id))
thresholdValue = 900; # This is the BiG-SLiCE Threshold that I used, which queried BGCs agaisnt teh 29,955 GCFs.
print('The threshold used for BiG-SLiCE processing: ' + str(thresholdValue))

strainsToRemoveB = "a1363|b482|b486"; # I remove these strains in the BiG-SLICE analysis because they failed QC.
print('Strains to remove from the BiG-SLiCE visualizations: ' + strainsToRemoveB)
    # Note: Look for this bit of code. You can modify this as you see fit.
    # remove a1363, b482, and b486 (if you want to remove c500, just add it to the string seen below "c500|a1363|b482|b486")
    # storageDF38 = storageDF1.copy(deep=True);
    # storageDF38 = storageDF38[storageDF38['Genome'].str.contains(strainsToRemoveB)==False]; # if the Genome column has these strings, remove those rows

# change genome name to be WMMC250 rather than c250: FIND THIS TEXT AND COMMENT IT IF YOU DON'T WANT WMM to be added before every strain name.
    # storageDF38['Genome'] = 'WMM' + storageDF38['Genome'];
    # storageDF38['Genome'] = [i.upper() for i in storageDF38['Genome']]

# Requires that the BiG-SLiCE runs be named:
    # "m6dfas.demux.bc1021_genome.contigs/ + 'thresh900_29955ranks_antismash511'"
    # So, an example one might be: "m6d.demux.bc1021_GENOMENAME.contigs/thresh900_29955ranks_antismash511"

# The relevant folders in BiG-SLiCE that you want to analyze.
allFolders = np.concatenate([np.arange(111,145,1), np.arange(149,157,1)]); # antismash v5.1.1: Right now, it looks at reports folders 111 to 144, and 150 to 156. The numbers can be found in the BiG-SLiCE visualization.
print('The report folders used for this analysis are: ')
print(allFolders)

# Path to csv file containing BGCs x GCFs (BiG-SLiCE): Used to minimize resource usage after running Section 5 once.
queriedBGC_GCFpath = 'C:\\Users\\imraa\\Documents\\UWM\\BugniLab\\Genome Assembly\\scriptOutputs\\antismash5-strainsQueried-rank.csv';

# Path to excel file containing the MIBIG BGCs and my BGCs, and their associated bgc product categories according to BiG-SCAPE classes (+ Hybrid (Not PKS/NRPS)):
bigSCAPEclassesAllpath = r'C:\Users\imraa\Documents\UWM\BugniLab\Yeast-Chemical-Genomics\annoCytoscape-v5-fullReplacedNames.xlsx';

# To make sure that the regenerated plots 4 and on match the publication exactly, re-order the data to match it.
# For some reason, this order is the one that was initially used, and I've stuck with it for my 38 Micromonosporaceae.
reOrderedIndex = r'C:\Users\imraa\Documents\UWM\BugniLab\Yeast-Chemical-Genomics\reOrderedIndex.csv';
orderedValues = pd.read_csv(reOrderedIndex);
orderedValues = orderedValues['Order Values:'].values.tolist(); # This is a list of BGC names. 779 of them. Otherwise, the plots are changed.

# Path to the excel file containing 5 columns: Needed for Figure 8
    # Genome name, Genera of the BGC, the BGC name, the BiG-SLiCE distance, and the Normalized Distance.
bactStatspath = r'C:\Users\imraa\Documents\UWM\BugniLab\Yeast-Chemical-Genomics\Micromonosporaceae-BGC-Genera-Python.xlsx';


#%% Section 4: Data Pre-Processing (antiSMASH)

chdir(antiSMASHdirectory); # change the directory the antiSMASH directory specified previously.
wd = getcwd(); # get the current path, it should match the antiSMASh directory
# check if the directories match
if wd == antiSMASHdirectory:
    print('Currently in the antiSMASH directory...')

asmStorage = {};
htmlStorageFull = []; # container for the html file path names
htmlStorage = []; # container for mini html file path names
genomeNameS = []; # container for the individual genome names
titleStorage = [];

for filename in os.listdir(wd): # look at each folder/file within the main folder
    newF = os.path.join(wd, filename); # store the folder/file name, append, so we can access further
    genomeNameS.append(newF); # store genome names "generally speaking" here.
    for filename2 in os.listdir(filename): # look at each folder/file in the subfolder
        if filename2.endswith('x.html'): # find only the index.html file 
            fname = os.path.join(newF, filename2) # find the filename/path
            htmlStorageFull.append(fname); # store the file name path
            htmlStorage.append(os.path.join(filename, filename2));
            # print("Current file name ..", os.path.abspath(fname)) # print statement to see if we found the right files

# antiSMASH v5.1.1 specific processing

counter = 0; # useful counter
print('Iterating through ' + str(len(htmlStorage)) + ' folders in the antiSMASH directory...')
for d in htmlStorage: # for each html file found
    htmlfile = open(d) # don't forget to close it at the very end when we're done with it
    contents = htmlfile.read(); # read it out
    beautifulSoupT = BeautifulSoup(contents, 'html.parser') # can update to a later parser eventually
    titleStorage.append(beautifulSoupT.head.title); # this contains the # of regions found, might be useful
    
    regList = list(beautifulSoupT.find_all('tbody')); # only take the first one
    regList = list(regList[-1]); # this contains all the info we could want
    
    asmStorageT = []; # storage for a singular html files import values (all regions)
    
    for i in np.arange(1, len(regList), 2): # 0, 2, 4,... all are empty headers '\n'
        poolT = regList[i].text.splitlines(); # get only the names, remove the \n (keeps spaces and +)
        asmStorageT.append(poolT); # take the split characters, put the list into the bigger list
    genomeInfo = pd.DataFrame(asmStorageT); # convert the big list into a dataFrame
    # this section is hardcoded, very bad idea, but works for now
    genomeInfo = genomeInfo.drop(columns = [0,1,3,4,6,9]); # remove the columns that are consistently bad under my current naming scheme
    genomeInfo.columns = ['Region', 'Type', 'From', 'To'];
    nameP = genomeNameS[counter].split('\\')[-1]; # Assuming that the folder name is: '\path\to\folder\genomeName', this will take just the 'genomeName'
    # nameP1 = genomeNameS[counter].replace('-','_').split('_')[-1].split('.')[0];
    # nameP = nameP1.split('.')[-2].split('_')[1]; # this is hardcoded, but basically takes just the genome name (c500, etc)
    
    # nameP = genomeNameS[counter].split('.')[-2].split('_')[1]; # this is hardcoded, but basically takes just the genome name (c500, etc)
    asmStorage[nameP] = genomeInfo; # adds to dict the key (genome name), and the dataframe (genome info)
    htmlfile.close(); # close the file
    counter += 1

# If there was any contamination that you are able to identify in your genome, and want to filter it out from the data here.
    # Example: If there's contamination in my bacteria strain d975, and I found that it was in antiSMASH regions 25 through 38 (counting down from the top in antismash), this is how I would remove those data pieces.
asmStorage['d975'].drop(list(np.arange(25,39,1)),inplace=True); # Because python is 0 indexed, region 1 is actually the 0th region in python.

print('Finished iterating through the ' + str(counter) + ' folders in the antiSMASH directory, see asmStorage dictionary for output...')
asmStorageTemp = asmStorage.copy();

# ---- To generate the antiSMASH annotation to BiGSCAPE class excel file:

stList = list(asmStorageTemp.keys());
unValST = {}; # will be a dictionary of genomes dataframes, each dataframe containing only the "Type" from the previous work
productList = list();
allValList1 = list();
fullListTypes = list();
for i in stList:
    valList1 = list();
    for j in asmStorageTemp[i]['Type']:
        individualProducts = j.split(',');
        productList.append(individualProducts);
        fullListTypes.append(j);
        if len(individualProducts) > 1:
            for b in individualProducts:
                valList1.append(b);
                allValList1.append(b);
        else:
            valList1.append(individualProducts[0]);
            allValList1.append(individualProducts[0]);
    strainTypeList = pd.DataFrame(valList1, columns = ['Type']);
    unValST[i] = strainTypeList;
# Count them:
uniqueVal2 = {};
indKeyList2 = list(unValST.keys());
for i in indKeyList2:
    uniqueVal2[i] = unValST[i].value_counts();

# Get the unique instances:
# Get all unique instances (counted) (in the aggregate): This takes every individual Type reported by antiSMASH. So, if a region has 4 antiSMASH predictions for it, it will treat those as individual 1s. DO NOT RECOMMEND USING.
allValDF = pd.DataFrame(allValList1, columns = ['Type']);
uniqueAll = allValDF.value_counts(); # this contains all unique instances & their numbers
# save the unique instances for excel manipulation
# uniqueAll.to_excel(resultsDirectory+r'\allUniqueTypes20220516.xlsx');

# Get all unique instances (counted) (in the aggregate) but without separating out each individual string: If a region has 4 antismash predictions, it will treat that region as 1 instance. USED FOR BiG_SCAPE CLASS ANNOTATION.
fullTypeDF = pd.DataFrame(fullListTypes);
fullTypeDFcounts = fullTypeDF.value_counts();
# UNCOMMENT IF YOU WANT AN EXCEL FILE WITH COLUMN 1 BEING THE UNIQUE ANTISMASH ANNOTATIONS. NECESSARY FOR MAPPING BIGSCAPE CLASSES TO ANTISMASH ANNOTATIONS.
# fullTypeDFcounts.to_excel(resultsDirectory+r'\allUniqueStringTypes.xlsx')

#%% Figure 1A: Generating the output used to annotate the ITOL Tree.

# ---- Data Processing.

print('Loading the excel file to convert the antiSMASH annotations to BiG-SCAPE classes...')
renameFILE = renamedAntiSMASHtypes_to_bigSCAPEclasses; # See Section 3 for information on how to generate this excel file.
renameDict = {};
# Create a dictionary the old fashioned way (manually writing each one): This maps each unique antiSMASH region of products to their associated BiG-SCAPE classes.
for index, row in renameFILE.iterrows():
    values12 = row['Type:'];
    values21 = row['SuperCluster:'];
    renameDict[values12] = values21;
print('Processing...')
# replace all the strings in Type with their mapped elements in the dictioary from renamedAntiSMASHtypes_to_bigSCAPEclasses
# Now asmStorageTemp has the Types in the traditional antismash annotations. asmStorageClean has the Types in the BiG-SCAPE classes.
asmStorageClean = deepcopy(asmStorageTemp);
keyList = list(asmStorageClean.keys());
counter1 = 0;
for i in keyList: # access each dataframe
    # print(i);
    counter1 +=1;
    # print(counter1);
    dFClean = asmStorageClean[i];
    testNew = [renameDict[b] for b in dFClean.Type];
    dFClean.Type = testNew;
# get a dict of unique entries (separated by each genome)
unqEntries = {};
for i in keyList: # access each dataframe
    ele1 = asmStorageClean[i].Type.value_counts();
    unqEntries[i] = ele1;
# create a super dataframe containing all the unique entries.
unqList1 = list(); # this list will contain ever Series (in the order of list(unqEntries.keys()))
for i in list(unqEntries.keys()):
    unqList1.append(unqEntries[i]);
unqDataFrame = pd.concat(unqList1, axis = 1); # create the dataframe, bunch of NaNs
unqDF = unqDataFrame.fillna(0); # turn the NaNs into 0s.
unqDF = unqDF.set_axis(list(unqEntries.keys()), axis = 1, inplace=False); # rename the columns into the proper names of the genomes

# Create a dataframe where any Type1PKS and NRPS are merged together into PKS/NRPS according to BiG-SCAPE, and the Hybrid (Not PKS/NRPS) class is created for any 2 or more BiG-SCAPE classes that aren't a T1PKS and NRPS.
unqDF2 = unqDF.copy(deep=True);
indList = list(unqDF.index);
unqDFShape = unqDF.shape;
unqDFr = unqDFShape[0];
unqDFh = unqDFShape[1];

# Find the antiSMASH product types that were converted to BiG-SCAPE classes, satisfy the constraints of those categories.
currentClasses = unqDF.index.tolist();
importantClasses = ['PKS other', 'RIPP', 'PKS', 'Terpene', 'Siderophore', 'PKS/NRPS', 'NRPS', 'Saccharides', 'Other'];
matchedInd = [i for i, value in enumerate(currentClasses) if value in importantClasses];
# Find the multiple antiSMASH product types that will be merged into a larger category called "Hybrid (Not PKS/NRPS)"
hybridList1 = [i for i in range(0,len(currentClasses)) if i not in matchedInd];

hybridCount = list(); # everything that is a hybrid (that isn't PKS/NRPS) gets combined together.
for column in unqDF2:
    hybridCount.append(sum(unqDF2[column][hybridList1]));
unqDF2hybrids = [unqDF2.index.tolist()[i] for i in hybridList1]
unqDF2.drop(unqDF2hybrids, inplace=True); # drop the hybrid classes (we will add them in later as a larger category)
hybridDF = pd.DataFrame(hybridCount); # convert the list of hybrid values (count) into a dataframe.
# unqDF.append(hybridCount)
    
unqDF3 = unqDF2.append(pd.DataFrame([hybridCount], columns = list(unqDF2.columns)), ignore_index = True);
indDF2N = list(unqDF2.index);
indDF2N.append('Hybrid (Not PKS/NRPS)');
# Manually replace 'PKS' with 'PKS 1'.
indDF2N[indDF2N.index('PKS')] = 'PKS 1'; # replacing 'PKS' with 'PKS 1' to further distinguish PKS 1 from PKS other
indDF2N[indDF2N.index('PKS other')] = 'PKS Other'; # replace PKS other with PKS Other
unqDF3.index = indDF2N;

# ---- Plot Generation
print('Plotting the stacked bar plot of antiSMASH products converted to BiG-SCAPE classes...')
unqDF3 = unqDF3.T # transpose.
# Remove the strains from Section 3.
unqDF3.drop(strainsToRemove, inplace=True);
# Re-order strains to match Section 3.
reOrderList.reverse();
unqDF3 = unqDF3.reindex(reOrderList); # re-order the strains in unqDF3 to match the order I want.

# Plot: Figure 1A antiSMASH annotations
# Remove A1363, B482, and B486 because they failed QC.

sns.set_context("poster", font_scale = 3)
sns.set_style("whitegrid")
colors = ['#a6cee3','#1f78b4','#b2df8a','#33a02c','#fb9a99','#e31a1c','#fdbf6f','#ff7f00','#cab2d6','#6a3d9a']; # colors match the ITOL TREE
classes1 = ['Terpene', 'PKS 1', 'RIPP', 'PKS Other', 'Siderophore', 'PKS/NRPS', 'NRPS', 'Other', 'Saccharides', 'Hybrid (Not PKS/NRPS)'];
unqDF4 = unqDF3.copy(deep=True);
unqDF4 = unqDF4.reindex(columns=classes1);
barP1 = unqDF4.plot(kind = 'barh', stacked=True, figsize = (100,80),color=colors);
barP1.xaxis.grid(False);
barP1.yaxis.grid(False);
plt.legend(fontsize=90)
# plt.savefig(resultsDirectory+r'\Figure1-A.pdf',dpi=300) # UNCOMMENT TO SAVE THIS BAR PLOT.
plt.show()
# UNCOMMENT BELOW To Generate the CSV file with values that can be put into the iTOL tree. The specific text necessary for the file can be found on the ITOL help pages. This is purely the antiSMASH annotation values.
# unqDF4 = unqDF4.astype(int);
# unqDF4.to_csv(resultsDirectory+r'\Figure1-A-antiSMASHannotations.csv');

#%% Figure 1B: Generating the pie plot.

print('Plotting the Pie Plot...')
# Get the total # of each BGC Class (BiG-SCAPE), make a pie chart using it. Do I include Streptomyces or not? I don't think so.
unqDFBGC = unqDF4.copy(deep=True);
# disable if you want the Pie Chart to include information with c500 the streptomyces
# unqDFBGC = unqDFBGC.drop(['c500']);
newDictBGC1 = {};
for i in list(unqDFBGC): # iterate through each column
    newDictBGC1[i] = [unqDFBGC[i].sum()]
newDFBGC = pd.DataFrame.from_dict(newDictBGC1);
newDFBGC = newDFBGC.T;
newDFBGC.set_axis(['BGC Count'],axis=1,inplace=True);
sns.set('poster');
sns.set(rc={'figure.figsize':(50,35)});
posterInfo = sns.plotting_context('poster');
sns.set(rc=posterInfo);
sns.set_style("whitegrid")
# colors = sns.color_palette('pastel')[0:10]; # change colors to seaborn color palette
colors = ['#a6cee3','#1f78b4','#b2df8a','#33a02c','#fb9a99','#e31a1c','#fdbf6f','#ff7f00','#cab2d6','#6a3d9a']; # colors match the ITOL TREE
sns.set(font_scale=4)
# sns.set_context(rc={"axes.labelsize":60})
def make_autopct(values): # allows for conversion from percentage to the actual value.
    def my_autopct(pct):
        total = sum(values)
        val = int(round(pct*total/100.0))
        # return '{p:.2f}%  ({v:d})'.format(p=pct,v=val) # original: returns X% (Y)
        return 'n={v:d}'.format(v=val)
    return my_autopct
['Terpene', 'PKS 1', 'RIPP', 'PKS other', 'Siderophore', 'PKS/NRPS','NRPS', 'Other', 'Saccharides', 'Hybrid (Not PKS/NRPS)']; # list of categories
myexplode=[0,0,0,0,0,0,0,0,0,0.1]
sel = newDFBGC.plot.pie(y='BGC Count', labels=None, autopct=make_autopct(newDFBGC['BGC Count']), explode=myexplode, shadow=False, pctdistance=1.1, labeldistance=1.1,colors = colors) # can modify font size of pie chart content (not legend) by passing: ,textprops={'fontsize':40} into the plot function.
sel.set_ylabel(None);
sel.legend_.remove();
# plt.savefig(resultsDirectory+r'\Figure1-B-piePlot.tiff', dpi=300) # has to be before plt.show(), saves to wrking directory
plt.show()

#%% Section 5: Data Pre-Processing (BiG-SLiCE) for Figure 3.

# ---- Processing necessary for Figure 3 and onwards.
print('Now working on BiG-SLiCE data pre-procesing...')
connMib = sqlite3.connect(pathToDataDB);
currMib = connMib.cursor();
# get parameters
run_id; # set in Section 3:
sqlPointer1 = currMib.execute("SELECT id,threshold FROM clustering WHERE run_id=" + str(run_id));
clusterAndThreshold = sqlPointer1.fetchall();
clustering_id, threshold = clusterAndThreshold[0][0], clusterAndThreshold[0][1]
# I can find all the BGC names for MIBIG:
sqlPointerMibig = currMib.execute("SELECT name FROM bgc WHERE dataset_id=1");
sqlPointerMibig1 = [i[0] for i in sqlPointerMibig]; # get the BGC names for the MIBIG entries
difVal = 200946;
filteredMibValsStorage = {};
filteredMibGCFidStorage = {};
counter = 0;
for i in sqlPointerMibig1:
    mibValues = pd.read_sql("SELECT gcf_membership.gcf_id, bgc.id, bgc.name, gcf_membership.membership_value FROM bgc, gcf_membership WHERE bgc.name="+"'" + i + "'" + " AND gcf_membership.bgc_id=bgc.id AND gcf_membership.rank = 0 ORDER BY gcf_membership.membership_value ASC", connMib);
    # get everything above 0 and below 900 (less than or equal to 900 is correct)
    filteredMibVals = mibValues[(mibValues.iloc[:,3] > 0) & (mibValues.iloc[:,3] <= thresholdValue) & (mibValues.iloc[:,0] >= 200946) & (mibValues.iloc[:,0] <= 230900)];
    if not(filteredMibVals.empty):
        filteredMibVals = filteredMibVals.sort_values(['gcf_id']);
        filteredMibGCFid = filteredMibVals.iloc[0].gcf_id; # GCF Number (the webpage one)
        filteredMibValsStorage[i] = filteredMibVals;
        filteredMibGCFidStorage[i] = filteredMibGCFid; # dict of BGC and the webpage 127.0.0.1:800/run/6/gcf/X where X is the value listed
    counter += 1;
unqCounts = list(set(list(filteredMibGCFidStorage.values()))); # goes from 1293 BGCs to 533 unique GCFs
unqDict = {};
for i in unqCounts:
    gcf_accession = currMib.execute("SELECT id_in_run FROM gcf, clustering WHERE gcf.clustering_id=4 and gcf.id=" + str(i) + " AND clustering.run_id=4");
    unqDict[str(i)] = gcf_accession.fetchall()[0];
    
unqList = list(unqDict.values());
unqList = [i[0] for i in unqList];
# unqList = [int(i) for i in unqList];
    
# I now have: the GCFs that the MiBiGs belong to.
# I need to get: the GCFs that my queried stuff belongs to.  WORKS
connMib = sqlite3.connect(pathToDataDB);
currMib = connMib.cursor();
# allFolders = np.concatenate([np.arange(49,77,1), np.arange(92,100,1), np.arange(101,107,1)]); # reports 49 to 76, 92 to 99, 101 to 106 (antimash v6.something)
# allFolders = np.concatenate([np.arange(111,145,1), np.arange(149,157,1)]); # antismash v5.1.1

# Version Below: For runs that were not run using a shell script (and have a more precise naming scheme)
# storageDict = {};
# for j in allFolders: # whatever the report runs i want to look at are
#     print(j)
#     connF = sqlite3.connect("C:/Users/imraa/Downloads/full_run_result/full_run_result/reports/" + str(j) + "/data.db") # open a connection
#     curF = connF.cursor();
#     dfGCF = pd.read_sql_query("SELECT * FROM gcf_membership", connF);
#     dfSliceGCF = dfGCF[(dfGCF.iloc[:,3] == 0)]; # get only the highest rank values
#     listBGC = list(dfSliceGCF.gcf_id);
#     numBGCs = len(dfGCF.bgc_id.value_counts());
#     list1, list2, list3, list4 = list(), list(), list(), list();
#     count12 = 0;
#     for i in np.arange(1,numBGCs+1,1):
#         name1 = curF.execute("SELECT orig_filename FROM bgc WHERE id=" + str(i)).fetchall()[0];
#         dist1 = curF.execute("SELECT membership_value, gcf_membership.gcf_id FROM gcf_membership WHERE bgc_id=" + str(i) + " AND rank=0").fetchall()[0];
#         gcf1 = currMib.execute("SELECT id_in_run FROM gcf, clustering WHERE gcf.clustering_id=4 AND gcf.id=" + str(dist1[1]) + " AND clustering.run_id=4").fetchall()[0];
#         list1.append(name1[0]);
#         list2.append(dist1[0]);
#         list3.append(dist1[1]);
#         list4.append(gcf1[0]);
#         count12 += 1;
#     genomeName = name1[0].split('.')[0];
#     nameDistDict = {'BGC': list1, 'Distance': list2, 'gcf_ID': list3, 'GCF_Value':list4};
#     nameDist = pd.DataFrame(nameDistDict);
#     storageDict[genomeName] = nameDist;
#     connF.close();

# Version Below: For runs that were run using a shell script and now have a name in the form of "m6dfas.demux.bc1021_genome.contigs/ + 'thresh900_29955ranks_antismash511'"
storageDict = {};
for j in allFolders: # whatever the report runs i want to look at are
    print('Analyzing folder: ' + str(j))
    connF = sqlite3.connect("C:/Users/imraa/Downloads/full_run_result/full_run_result/reports/" + str(j) + "/data.db") # open a connection
    curF = connF.cursor();
    dfGCF = pd.read_sql_query("SELECT * FROM gcf_membership", connF);
    dfSliceGCF = dfGCF[(dfGCF.iloc[:,3] == 0)]; # get only the highest rank values
    listBGC = list(dfSliceGCF.gcf_id);
    numBGCs = len(dfGCF.bgc_id.value_counts());
    list1, list2, list3, list4 = list(), list(), list(), list();
    count12 = 0;
    for i in np.arange(1,numBGCs+1,1):
        name1 = curF.execute("SELECT name FROM bgc WHERE id=" + str(i)).fetchall()[0]; # modified for this new version
        dist1 = curF.execute("SELECT membership_value, gcf_membership.gcf_id FROM gcf_membership WHERE bgc_id=" + str(i) + " AND rank=0").fetchall()[0];
        gcf1 = currMib.execute("SELECT id_in_run FROM gcf, clustering WHERE gcf.clustering_id=4 AND gcf.id=" + str(dist1[1]) + " AND clustering.run_id=4").fetchall()[0];
        name2 = name1[0].split('/')[0]; # genome name folder
        name22 = name1[0].split('/')[1]; # tig.region
        if name2 != 'm6_a1363': # hard code it because i messed up.
            name3 = name2.split('.')[-2]; # bc_genomeName
            if len(name3.split('_')) < 2: # a rule for if its bc12_genomeName or bc-genomeName
                name4 = name3.split('-')[-1];
                name42 = name4 + '.' + name22;
            else:
                name4 = name3.split('_')[-1]; # just genomeName
                name42 = name4 + '.' + name22; # genomeName.tig00.region00
        else:
            name4 = 'a1363';
            name42 = name4 + '.' + name22;
        list1.append(name42);
        list2.append(dist1[0]);
        list3.append(dist1[1]);
        list4.append(gcf1[0]);
        count12 += 1;
    genomeName = name4; # name1[0].split('.')[0];
    nameDistDict = {'BGC': list1, 'Distance': list2, 'gcf_ID': list3, 'GCF_Value':list4};
    nameDist = pd.DataFrame(nameDistDict);
    storageDict[genomeName] = nameDist;
    connF.close();

# REMOVE THE UPCOMING BLOCK IF YOU ARE RUNNING YOUR OWN ANALYSIS.
# Remove the extra stuff in WMMD975 that might be contamination.
d975Stuff = storageDict['d975'];
d975StuffKeep = d975Stuff[d975Stuff['BGC'].str.contains('tig00000001')];
d975StuffRemove = d975Stuff[~d975Stuff['BGC'].str.contains('tig00000001')];
storageDict['d975'] = d975StuffKeep;    

storageF = {};
for i in list(storageDict.keys()):
    dfInt = storageDict[i].copy();
    underThresh = dfInt[(dfInt.iloc[:,1] <= thresholdValue)]; # slice the genome dataframe to only take rows with a distance of 900 or less
    uThresh = list(underThresh.GCF_Value);
    ifInList = [i in unqList for i in uThresh];
    indValues = list(underThresh.index);
    dfInt.loc[:,4] = False;
    # underThresh['Found_In_MIBIG_Associated_GCF'] = ifInList;
    for j in np.arange(0, len(ifInList),1):
        dfInt.loc[indValues[j],4] = ifInList[j]; # for the indexes found to be below the threshold, plop in the True/False as to whether or not they were found in the MIBIG GCFs
    dfInt.columns = ['BGC', 'Distance', 'GCF_ID', 'GCF_Value', 'Found_MIBIG_GCF']; # rename columns
    storageF[i] = dfInt;
        
# storageF now contains a True/False indicator for MIBIG relationship (based on distance from GCF)
storageG = storageF.copy();
storageH = {};
for i in list(storageG.keys()):
    dfInt1 = storageG[i].copy();
    undThresh = dfInt1[(dfInt1.iloc[:,1] <= thresholdValue)];
    storageH[i] = undThresh;
# StorageH only contains rows where the Distance was below the threshold of 900

storageHOpposite = {}; # only get rows where the Distance is above the threshold 900
for i in list(storageG.keys()):
    dfInt2 = storageG[i].copy();
    undThreshO = dfInt2[~(dfInt2.iloc[:,1] <= thresholdValue)];
    storageHOpposite[i] = undThreshO;
    
# count how many rows there are total that are above the threshold of 900 (minimum) (318 for the original 25 genomes)
counterO = 0;
for i in list(storageHOpposite.keys()):
    counterO = counterO + len(storageHOpposite[i]);
    
# count # of rows below 900    
counterL = 0;
for i in list(storageH.keys()):
    counterL = counterL + len(storageH[i]);    

counterM = 0; # total # of BGCs found.
for i in list(storageF.keys()):
    counterM = counterM + len(storageF[i]);
    
counterN = 0; # of the BGCs that were clustered (sub-threshold), how many were associated with a MIBIG-associated GCF
for i in list(storageH.keys()): # storageH contains only BGCs that had a distance of less than 900.
    temporaryDataFrame = storageH[i];    
    counterN = counterN + len(temporaryDataFrame.loc[temporaryDataFrame['Found_MIBIG_GCF'] == True]);

# Everything above is all the underlying data you need to generate:
    # The horizontal/vertical MIBIG plots.
    # The violin + catplot.
# This does not include all the info necessary to generate the distance plots.

# Sort the data by the Distance.
for b in list(storageF.keys()):
    storageF[b].sort_values(by=['Distance'], inplace=True);
    
# Make a super dataframe.
storageDFNew = pd.concat(storageF.values(), ignore_index = True);
storageDFNew.sort_values(by=['Distance'], inplace=True);

# Add additional columns, regarding whether the BGC-GCF is associated/unassociated with a MIBiG-BGC
nameVals = [i.split('.')[0] for i in storageDFNew.BGC]; # get the genome name from the BGC column
storageDF1 = storageDFNew.copy(deep=True); # deep copy, doesn't modify
storageDF1.BGC = nameVals; # replace all the unique regions with the genome name
storageDF1.columns = ['Genome', 'Distance', 'GCF_ID', 'GCF_Value', "Biosynthetic Gene Clusters (BGCs)"]
storageDF1.loc[storageDF1["Biosynthetic Gene Clusters (BGCs)"] == False, "Biosynthetic Gene Clusters (BGCs)"] = 'Unassociated with a MiBiG-GCF';
storageDF1.loc[storageDF1["Biosynthetic Gene Clusters (BGCs)"] == True, "Biosynthetic Gene Clusters (BGCs)"] = 'Associated with a MiBiG-GCF';

# include information on < or > 900 as well as MIBIG association.
storageDF1.loc[:,5] = False;
newList1 = list(storageDF1[(storageDF1.iloc[:,1] <= thresholdValue) & (storageDF1.iloc[:,4] == 'Associated with a MiBiG-GCF')].index);
for j in np.arange(0, len(newList1),1):
    storageDF1.loc[:,(5)][newList1[j]] = "MIBIG-Associated"; # for the indexes found to be below the threshold, plop in the True/False as to whether or not they were found in the MIBIG GCFs
newList2 = list(storageDF1[(storageDF1.iloc[:,1] <= thresholdValue) & (storageDF1.iloc[:,4] == 'Unassociated with a MiBiG-GCF')].index);
for j in np.arange(0,len(newList2),1):
    storageDF1.loc[:,(5)][newList2[j]] = "Not MIBIG-Associated"; # for the indexes found to be below the threshold, plop in the True/False as to whether or not they were found in the MIBIG GCFs
newList3 = list(storageDF1[(storageDF1.iloc[:,1] > thresholdValue)].index);
for j in np.arange(0,len(newList3),1):
    storageDF1.loc[:,(5)][newList3[j]] = "No Information"; # for the indexes found to be above the threshold, plop in the True/False as to whether or not they were found in the MIBIG GCFs
storageDF1.columns = ['Genome', 'Distance', 'GCF_ID', 'GCF_Value', "Biosynthetic Gene Clusters (BGCs)", 'Color'];


# REMOVE THIS IF YOU ARE RUNNING YOUR OWN ANALYSIS
# remove a1363, b482, and b486 (if you want to remove c500, just add it to the string seen below "c500|a1363|b482|b486")
storageDF38 = storageDF1.copy(deep=True);
storageDF38 = storageDF38[storageDF38['Genome'].str.contains(strainsToRemoveB)==False]; # if the Genome column has these strings, remove those rows
# change genome name to be WMMC250 rather than c250:
storageDF38['Genome'] = 'WMM' + storageDF38['Genome'];
storageDF38['Genome'] = [i.upper() for i in storageDF38['Genome']]

#%% Figure 3: Requires Sections 3 and 5.

print("Plotting the scatter plot of genomes to queried BGC distance in BiG-SLiCE...")
plt.figure(figsize=(100,90));
sns.set_context('poster', font_scale=3, rc={'xtick.labelsize':66});
sns.set_style("whitegrid")
g= sns.stripplot(x='Genome', y='Distance',hue='Color', data=storageDF38, s = 30)#.set(title='MiBiG-Associated GCFs of BiG-SLiCE Queried Genome BGCs');
plt.axhline(900, 0, 1, linewidth=10, color='r')
# g.legend_.set_title(None)
# g.legend_.remove() # remove the legend entirely
g.xaxis.grid(True);
g.yaxis.grid(True);
plt.xlabel('Genomes', fontsize = 100);
plt.ylabel('Distance', fontsize = 100)
# g.set(xlabel=None, ylabel=None) # remove the x axis label and the y axis label
g.set(ylim=(-500,4500));
plt.xticks(rotation=90) # rotate x labels to be vertical
# plt.savefig(resultsDirectory+r'\Figure 3.pdf',dpi=300)
plt.show()

#%% Section 6: Data Pre-Processing (BiG-SLiCE) for Figure 4. (Memory-Intensive)

# Computationally-intensive. (Generates a BGC x GCF dataframe)
connMib = sqlite3.connect(pathToDataDB);
currMib = connMib.cursor();
# allFolders = np.concatenate([np.arange(111,145,1), np.arange(149,157,1)]); # antismash v5.1.1 # DEFINED IN SECTION 3
storageDict = {};
counterN=0;
for j in allFolders: # whatever the report runs i want to look at are
    print('Analyzing folder: ' + str(j) + ' and scraping all queried distances associated with this strain...')
    connF = sqlite3.connect(pathToReportsDB + str(j) + "/data.db") # open a connection
    curF = connF.cursor();
    dfGCF = pd.read_sql_query("SELECT * FROM gcf_membership", connF);
    dfSliceGCF = dfGCF[(dfGCF.iloc[:,3] == 0)]; # get only the highest rank values
    listBGC = list(dfSliceGCF.gcf_id);
    numBGCs = len(dfGCF.bgc_id.value_counts());
    list1, list2, list3, list4 = list(), list(), list(), list();
    count12 = 0;
    for i in np.arange(1,numBGCs+1,1):
        name1 = curF.execute("SELECT name FROM bgc WHERE id=" + str(i)).fetchall()[0]; # modified for this new version
        dist1 = curF.execute("SELECT membership_value, gcf_membership.gcf_id FROM gcf_membership WHERE bgc_id=" + str(i)).fetchall(); # modded to grab all ranks
        # gcf1 = currMib.execute("SELECT id_in_run FROM gcf, clustering WHERE gcf.clustering_id=4 AND gcf.id=" + str(dist1[1]) + " AND clustering.run_id=4").fetchall()[0];
        name2 = name1[0].split('/')[0]; # genome name folder
        name22 = name1[0].split('/')[1]; # tig.region
        # I HARDCODED THIS SECTION because it was easier. 
        # What this does is it scrapes the genome name associated with this run, and then the contig and the region for the BGC. You can re-implement this how you see fit.
        if name2 != 'm6_a1363': # hard code it because i messed up.
            name3 = name2.split('.')[-2]; # bc_genomeName
            if len(name3.split('_')) < 2: # a rule for if its bc12_genomeName or bc-genomeName
                name4 = name3.split('-')[-1];
                name42 = name4 + '.' + name22;
            else:
                name4 = name3.split('_')[-1]; # just genomeName
                name42 = name4 + '.' + name22; # genomeName.tig00.region00
        else:
            name4 = 'a1363';
            name42 = name4 + '.' + name22;
        for k in np.arange(0,len(dist1),1):
            gcf1 = currMib.execute("SELECT id_in_run FROM gcf, clustering WHERE gcf.clustering_id=4 AND gcf.id=" + str(dist1[k][1]) + " AND clustering.run_id=4").fetchall()[0];
            list1.append(name42);
            list2.append(dist1[k][0]);
            list3.append(dist1[k][1]);
            list4.append(gcf1[0]);
        count12 += 1;
    counterN +=1;
    print('Completed folder: ' + str(counterN) + ' of ' + str(len(allFolders)))
    genomeName = name4; # name1[0].split('.')[0];
    nameDistDict = {'BGC': list1, 'Distance': list2, 'gcf_ID': list3, 'GCF_Value':list4};
    nameDist = pd.DataFrame(nameDistDict);
    storageDict[genomeName] = nameDist;
    connF.close();

# REMOVE IF ERROR (Hardcoded for my own data)
# Remove the extra stuff in WMMD975 that might be contamination.
d975Stuff = storageDict['d975'];
d975StuffKeep = d975Stuff[d975Stuff['BGC'].str.contains('tig00000001')];
d975StuffRemove = d975Stuff[~d975Stuff['BGC'].str.contains('tig00000001')];
storageDict['d975'] = d975StuffKeep;

print('Finished scraping all of the BiG-SLiCE report folders...')
print('Construcing file for exporting...')
# Merge all of the information into one big dataframe. (X, where X = GCF*BGC, by 4)
newDF = pd.DataFrame(columns = list(storageDict['a1363'].columns));
for i in list(storageDict.keys()):
    print(i); # Tracker: outputs to terminal
    oldDF = storageDict[i].sort_values(by=['BGC','gcf_ID']);
    newDF = pd.concat([newDF,oldDF], ignore_index=True);
testDF = newDF;

# Dataframe (rows are BGCs, columns are GCF values)
testT1 = list(testDF.BGC); # get the BGC names.
testT1 = list(set(testT1)); # get the unique BGC names.
testT2 = list(testDF.gcf_ID); # get the gcf id
testT2 = list(set(testT2)); # get the unique GCF ids

# Get the individual data into lists, ordered and sorted.
superList1 = [];
counterT = 0;
counterT2 = 0;
for i in testT1:
    placeHolder1 = testDF.loc[(testDF['BGC'] == i)]; # returns rows of only the BGC (sorted by gcf_ID)
    pH2 = list(placeHolder1.Distance);
    pH3 = list(placeHolder1.gcf_ID);
    superList1.append(pH2)
    counterT2 += 1;
    if (counterT2 - counterT) >= 30: # Tracker (outputs to console)
        # print(counterT2);
        counterT = counterT2;

print("Making a mega dataframe, takes awhile...")
# Make a super dataframe, with the columns as GCFs-200946 to the end.
superDF2 = pd.DataFrame([i for i in superList1], columns = testT2);
superDF2.index = testT1; # change the row names to be the BGC names.

superDFbackup = superDF2.copy(deep=True);

# REMOVE IF ERROR, BASED ON MY ORIGINAL DATA: Removes the Streptomycetaceae (c500) and the ones that failed QC.
superDF2 = superDF2[~superDF2.index.str.contains('c500|a1363|b482|b486')];

print('Exporting the BGC x GCF file for future re-use to ' + resultsDirectory + r'\antismash5-strainsQueried-rank.csv')

# Must swap the rows and columns, as excel doesn't like having 29,955 columns.
superDF2T = superDF2.T;

# Export the superDF2 (GCF x BGC file) to CSV for ease of access later.
superDF2T.to_csv(resultsDirectory+r'\antismash5-strainsQueried-rank.csv');

#%% Section 6.5: Additional Data Pre-Processing (BiG-SLICE).

# ---- If re-using a csv file containing all BGCs x GCFs queried, can load here.
print('Loading file at ' + queriedBGC_GCFpath)
# Load file.
superDF3 = pd.read_csv('C:\\Users\\imraa\\Documents\\UWM\\BugniLab\\Genome Assembly\\scriptOutputs\\antismash5-strainsQueried-rank.csv');
# Get the index.
superDF3.index = superDF3.iloc[:,0]; # replace index with GCFs
# Remove the first column (which previously contained the index)
superDF3.drop(columns = superDF3.columns[0], axis=1, inplace=True); # remove first column
# Transpose so it is in the shaep of BGC x GCF.
superDF3=superDF3.T;
# Scrape the index.
testT1 = list(superDF3.index);

# REPLACE THE INDEX: ONLY NECESSARY IF YOU ARE TRYING TO REGENERATE MY FIGURES
testT1 = orderedValues; # gets the order of indexes, as per my original data
superDF4 = superDF3.reindex(testT1); # remaps the rows to match my original data (keep superDF3 as a backup)

# SuperDF2 and SuperDF3 should be identical. If not, something's wrong.
# Can check using this code (uncomment)
# superDF2.equals(superDF4) # If True, they are identical.

superDF2 = superDF4; # 

# ---- Data necessary for the Figures.

print('Initializing various distance metrics to explore...')
superDict0 = {}; # dictionary of the different simplified datasets I want to play with.
superDict0['orig'] = superDF2;
dictDistMetrics = {}; # initialize empty dictionary for storage
# You can uncomment this line below to add more distance metrics.
# distMets = ['euclidean', 'cosine', 'cityblock', 'l1', 'l2', 'manhattan', 'braycurtis', 'canberra', 'chebyshev', 'correlation', 'hamming'] # this is where the different strings for distance metrics go
# I only wanted to use 'chebyshev' because that was the best performing one. Comment this if the previous line is uncommented.
distMets = ['chebyshev'] # this is where the different strings for distance metrics go
print('Distance metrics currently being used are: ' + ', '.join(distMets))
for i in distMets: # split according to distance metrics
    print('Generating distance matrix for: ' + i);
    dictDistMetrics[i] = {}; # initialize sub-dictionary (one for each distance metric)
    for j in list(superDict0.keys()): # split according to different simplified datasets
        outputDist0 = pd.DataFrame(sklearn.metrics.pairwise_distances(superDict0[j], Y=None, metric=i, n_jobs=None, force_all_finite=True), columns = testT1);
        outputDist0.index = testT1;
        dictDistMetrics[i][i+"_"+j] = outputDist0; # pop it back in
# Convert to array:
print('Converting distance matrices to arrays...')
dictArray = {};        
for i in distMets:
    dictArray[i] = {};
    for j in list(superDict0.keys()):
        outputArray = dictDistMetrics[i][i+"_"+j].to_numpy();
        dictArray[i][i+"_"+j] = outputArray;
# Perform MDS
dictMDS = {};        
nComps = 2; # number of components
for i in distMets:
    print('Performing multidimensional scaling using ' + str(nComps) + ' components for: ' + i)
    dictMDS[i] = {};
    for j in list(superDict0.keys()):
        mdsModelA = MDS(n_components = nComps, dissimilarity='precomputed', random_state=0);
        dataTrans = mdsModelA.fit_transform(dictArray[i][i+"_"+j]);
        dictMDS[i][i+"_"+j] = dataTrans;
        #dictMDS[i][i+"_"+j]
# Set up parameters for MDS plotting

# pio.renderers.default='browser' #interactive in browser popup (UNCOMMENT IF YOU LIKE BROWSER-BASED VISUALIZATION)
truncG = [i.split('.')[0] for i in testT1]; # assuming the BGCs are named: genome.contig~~~.region~~~, take the first part to find the genome names.
nGenomes = len(unique(truncG)); # Find the number of genomes used for the analysis.
nColors = px.colors.sample_colorscale('turbo', [n/(nGenomes -1) for n in range(nGenomes)]); # Generate nice colors.

# Try coloring the MDS plot using BiG-SCAPE category classes.

# NOTE: I took each individual MIBIG BGC that was associated with my genomes using BiG-SCAPE. I ran them through antiSMASH v5.1.1. Then I manually labeled them into the 10 category classes (PKS/NRPS and Hybrid are included). That way, all of my 779 BGCs and the MiBIG BGCs associated with them are all categorizable.
# Then I added them to the file containing all of my Micromonosporaceae BGCs, labeled in the same format.
bgSCAPEclasses = pd.read_excel(bigSCAPEclassesAllpath);
classList = [];
[classList.append(bgSCAPEclasses.loc[bgSCAPEclasses['name 1'] == i]['bgc product'].tolist()[0]) for i in testT1];
# Make sure I have max 10 classes (because that's how many I should have under my scheme)
nClasses = len(unique(classList));

colors = ['#a6cee3','#1f78b4','#b2df8a','#33a02c','#fb9a99','#e31a1c','#fdbf6f','#ff7f00','#cab2d6','#6a3d9a']; # colors match the ITOL TREE
classes1 = ['Terpene', 'PKS 1', 'RIPP', 'PKS Other', 'Siderophore', 'PKS/NRPS', 'NRPS', 'Other', 'Saccharides', 'Hybrid (Not PKS/NRPS)']; # matches the colors indices.

# Construct a dictionary of palettes.
palDict = {};
for i in range(0,len(colors),1):
    palDict[classes1[i]] = colors[i];

#%% Figure 4: Requires Sections 2, 3, 5, (6 OR 6.5)

# Make a dataframe using whichever values of data you want. This one is using chebyshev, can change to match your metrics of interest.
dfMDS = pd.DataFrame(dictMDS['chebyshev']['chebyshev_orig'], index = testT1)
dfMDS['Category'] = classList;
dfMDS['Genome'] = truncG;

# Set up the figure details.
plt.figure(figsize = (100,100));
sns.set_context("poster", font_scale = 4)
sns.set_style("whitegrid")
# Create the Facet Grid element.
g = sns.FacetGrid(dfMDS, col = 'Category', hue='Category', height=20, aspect=1, palette = palDict, col_wrap=5)
g = (g.map(plt.scatter, 0, 1, edgecolor='white', s = 600)) # Plot a scatter plot in each section, based on the Category split.
g = g.set_titles("{col_name}", size = 60) #size = 200) # Add relevant title names.
plt.subplots_adjust(top=0.92) # Give a little bit of space at the top to avoid overlapping title and superTitle.
g.set_xlabels('Coordinate 1',fontsize=60) # Axes.
g.set_ylabels('Coordinate 2',fontsize=60) # Axes.
for ax in g.axes.flat:
    _ = plt.setp(ax.get_yticklabels(), size = 40)
    _ = plt.setp(ax.get_xticklabels(), size = 40)
# Modify the title to match which Metric you're using and all that.
g = g.fig.suptitle('Multidimensional Scaling (Chebyshev) on 779 Micromonosporaceae BGCs, Split By Class', size = 80)
plt.tight_layout()
# Uncomment to save the figure to the Results directory.
# plt.savefig(resultsDirectory+r'\Figure 4.pdf',dpi=300) # has to be before plt.show(), saves to wrking directory
plt.show()

#%% Figure 5: Requires Section 2, 3, 5, (6 OR 6.5).
    # Specifically, it requires the storageDict dictionary, and the testT1 list, along with the other information.
    # PKS/NRPS annotated plot.

# Make a dataframe using whichever values of data you want. This one is using chebyshev, can change to match your metrics of interest.
dfMDS = pd.DataFrame(dictMDS['chebyshev']['chebyshev_orig'], index = testT1)
dfMDS['Category'] = classList;
dfMDS['Genome'] = truncG;
categoryList = dfMDS['Category'].unique().tolist();

# Create a dataframe to store the older information from storageDict (Distance & such).
storage2DF = pd.DataFrame();
# Remove low quality genomes & WMMC500:
    # Make a backup first.
    # IF ERROR: REMOVE (was designed to work with specifically my data)
storageDictBackup = copy.deepcopy(storageDict);
for i in ['a1363','b482','b486','c500']:
    storageDict.pop(i);

for i in list(storageDict.keys()):
    storage2DF = pd.concat([storage2DF,storageDict[i]])
# list comprehension (if/else statement). GOal: '> 900' if the BGC fell above 900 in BiGSLICE, otherwise '< 900'
# bigSLICEinfo = ['> 900' if storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] >= 900 else '< 900' for iter1 in testT1];
bigSLICEinfo = ['Did Not Cluster' if storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] >= 900 else 'Clustered' for iter1 in testT1];
# find the distance to the closest GCF
closestGCFDist = [storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] for iter1 in superDict0['orig'].index.tolist()];
# find the distance to the X closest GCFs, then average them. Where X is set above as aveDist
aveDist = 3;
aveDistStorage = [sorted(superDict0['orig'].loc[iter2])[:aveDist] for iter2 in superDict0['orig'].index]; # list of lists
averageDist = [np.mean(entry1) for entry1 in aveDistStorage]; # average each list inside the list of lists
dfMDSTemp2 = dfMDS.copy(deep='true'); # create a backup.
# Append additional columns to the dataFrame.
dfMDSTemp2['BIG-SLICE'] = bigSLICEinfo;
dfMDSTemp2['Distance'] = closestGCFDist;
dfMDSTemp2['Average Distance'] = averageDist;
dfMDSTemp2['# of GCF Distances used:'] = aveDist;

dfMDS2 = dfMDSTemp2.copy(deep=True) # remap to a new name.

# Plot PKS/NRPS figure.
plottedClass = 'PKS/NRPS';
curData = dfMDS2.loc[dfMDS2['Category'] == plottedClass]; # current cateogry (iterates through Saccharides, Hybrids, etc)
plt.figure(figsize=(100,80));
sns.set_context("poster", font_scale = 4)
sns.set_style("whitegrid")
g = sns.scatterplot(data=curData, x=0, y=1,hue='Average Distance' ,size = 'Average Distance',sizes = (2000,10000) , edgecolor='white'); # default hue
g.set(xlabel='Coordinate 1', ylabel='Coordinate 2') # 
plt.title('Multidimensional Scaling (Chebyshev): ' + plottedClass, fontsize = 150)
plt.xlim(-2500, 3000) # hardcoded right now, because i want all the plots to look identical
plt.ylim(-3500, 3500) # hardcoded, see above
plt.tight_layout()
# replace characters not accepted by windows into ones that are (for saving in batches purposes)
plottedClass = plottedClass.replace('/','-'); # replaces the "/" character with the "-" character
plottedClass = plottedClass.replace(' ','-'); # replaces the " " space with the "-" character
# plt.savefig(resultsDirectory+r'\Figure-5.pdf', dpi=300)
plt.show()

#%% Section 7: Data-Processing for Figures 6 and 7. Requires Sections 2,3,5, and (6 OR 6.5)

# Find which indices in the BGCs map to which classes.
classDict = {}; # Keys are the classes, each key links to a list of the indices where it was found in classList
for i in classes1:
    classDict[i] = [a for a, x in enumerate(classList) if x == i]; 
# Copy superDF2 just incase.
superCatDF2 = superDF2.copy(deep=True);

dictDistMetrics = {}; # initialize empty dictionary for storage
distMets = ['euclidean', 'cosine', 'cityblock', 'l1', 'l2', 'manhattan', 'braycurtis', 'canberra', 'chebyshev', 'correlation', 'hamming'] # this is where the different strings for distance metrics go
superDict0 = {}; # dictionary of the different simplified datasets I want to play with.
# Create the datasets (sliced by bigSCAPE categories)
for b in classes1: 
    newDF2 = superCatDF2.iloc[classDict[b]];
    superDict0[b+"_orig"] = newDF2;
# Initialize an empty dictionary
dictDistMetrics = {};
# Run the distance metrics, the structure of dictDistMetrics is as follows:
    # 'Terpene_orig' -> 'Terpene_orig_euclidean", "Terpene_orig_cosine", etc (each is a dataframe)
for j in list(superDict0.keys()):
    dictDistMetrics[j] = {};
    print('Iterating over BGC class: ' + j)
    for i in distMets:
        print('Computing distance for: ' + i)
        outputDist0 = pd.DataFrame(sklearn.metrics.pairwise_distances(superDict0[j], Y=None, metric=i, n_jobs=None, force_all_finite=True), columns = superDict0[j].index.tolist());
        outputDist0.index = superDict0[j].index.tolist();
        dictDistMetrics[j][j+"_"+i] = outputDist0; # pop it back in
# Convert the distance metrics from dataframes to arrays for MDS.
print('Converting dataframes to arrays...')
dictArray = {};          
for j in list(superDict0.keys()):
    dictArray[j] = {};
    for i in distMets:
        outputArray = dictDistMetrics[j][j+"_"+i].to_numpy();
        dictArray[j][j+"_"+i] = outputArray;  

# Perform MDS
print('Performing MDS...')
dictMDS = {};        
nComps = 2; # number of components
for j in list(superDict0.keys()):
    print('Iterating over BGC class: ' + j)
    dictMDS[j] = {};
    for i in distMets:
        print('Metric used for MDS currently: ' + i)
        mdsModelA = MDS(n_components = nComps, dissimilarity='precomputed', random_state=0);
        dataTrans = mdsModelA.fit_transform(dictArray[j][j+"_"+i]);
        dictMDS[j][j+"_"+i] = dataTrans;

# Remove low quality genomes & WMMC500:
    # Make a backup first.
    # IF ERROR: COMMENT OUT (was designed to work with specifically my data)
storageDictBackup = copy.deepcopy(storageDict);
for i in ['a1363','b482','b486','c500']:
    storageDict.pop(i);
# Need to include metadata based on BiG-SLiCE INFORMATION
# needs: storageDict
storage2DF = pd.DataFrame();
for i in list(storageDict.keys()):
    storage2DF = pd.concat([storage2DF,storageDict[i]])

# Convert em all to dataframes, add a column for the Category
print('Converting to dataframes for ease of usage...')
aveDist = 3;
dictDFMDS = {};
for j in list(superDict0.keys()):
    dictDFMDS[j] = {};
    for i in distMets:
        dfMDStemp = pd.DataFrame(dictMDS[j][j+"_"+i], index = superDict0[j].index.tolist())
        dfMDStemp['Category'] = j.split("_")[0]; # bigscape product types
        # grab the <900 >900 information from bigslice
        # list comprehension (if/else statement). GOal: '> 900' if the BGC fell above 900 in BiGSLICE, otherwise '< 900'
        bigSLICEinfo = ['> 900' if storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] >= 900 else '< 900' for iter1 in superDict0[j].index.tolist()];
        # find the distance to the closest GCF
        closestGCFDist = [storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] for iter1 in superDict0[j].index.tolist()];
        # find the distance to the X closest GCFs, then average them. Where X is set above as aveDist
        aveDistStorage = [sorted(superDict0[j].loc[iter2])[:aveDist] for iter2 in superDict0[j].index]; # list of lists
        averageDist = [np.mean(entry1) for entry1 in aveDistStorage]; # average each list inside the list of lists
        
        dfMDStemp['BIG-SLICE'] = bigSLICEinfo;
        dfMDStemp['Distance'] = closestGCFDist;
        dfMDStemp['Average Distance'] = averageDist;
        dfMDStemp['# of GCF Distances used:'] = aveDist;
        dictDFMDS[j][j+"_"+i] = dfMDStemp;

#%% Figure 6: Requires Sections 2,3,5,(6 OR 6.5), and 7.

specificMets = ['chebyshev']; # the chosen metric for Figure 6 was chebyshev.
# Future-proofing with a dictionary of dataframes.
dictDFMDS_Cat = {};
for j in specificMets:
    dfConCat = pd.DataFrame();        
    for i in list(dictDFMDS.keys()):
        dfConCat = pd.concat([dfConCat,dictDFMDS[i][i+'_'+j]])
        if i == list(dictDFMDS.keys())[-1]:
            dictDFMDS_Cat[j] = dfConCat;
            break # stop at the end
currentPlotMetric = specificMets[0]; # choose chebyshev as the metric for Figure 6.
plt.figure(figsize=(100,80));
sns.set_context("poster", font_scale = 4)
sns.set_style("whitegrid")
g = sns.relplot(data=dictDFMDS_Cat[currentPlotMetric], x=0, y=1, hue='Category', palette=palDict, col='Category', col_wrap=5, height=22, aspect=1, s=1000, edgecolor='white')#.set_title('Multidimensional Scaling (Chebyshev) on 779 Micromonosporaceae BGCs, Split By Class', size = 80)
for ax in g.axes.flat:
    _ = plt.setp(ax.get_yticklabels(), size = 90)
    _ = plt.setp(ax.get_xticklabels(), size = 90)
    ax.grid(True, axis='both') # re-enable gridlines to match my other plots
g = g.set_titles("{col_name}", size = 90) #size = 200)
g.set_xlabels('Coordinate 1',fontsize=90)
g.set_ylabels('Coordinate 2',fontsize=90)
g.legend.remove(); # remove legend
g.fig.suptitle('Multidimensional Scaling (' + 'Chebyshev' +') Across BGC Classes', size = 110) # 
plt.tight_layout(rect=[0,0.03,1,1]) # defining this stops the figure titles from overlapping with the overall title
# plt.savefig(resultsDirectory+r'\Figure-6.pdf', dpi=300) # has to be before plt.show(), saves to wrking directory
plt.show()

#%% Figure 7: Requires Sections 2,3,5,(6 OR 6.5), and 7.
    # Chebyshev distance metric, PKS/NRPS class, annotated.
            
# Define parameters.
spMet = 'chebyshev';
spClass = 'PKS/NRPS_orig';
# Plot the individual PKS/NRPS
plt.figure(figsize = (100,100));
sns.set_context("poster", font_scale = 9)
sns.set_style("whitegrid")
# BELOW: Scales the scatter plot based on average distance to closest 3 GCFs.
rel = sns.scatterplot(x=0, y=1, data= dictDFMDS[spClass][spClass+"_"+spMet],hue= 'Average Distance', size = 'Average Distance', sizes = (5000,15000), edgecolor='black');
rel.xaxis.grid(True);
rel.yaxis.grid(True);
rel.set(xlabel='Coordinate 1', ylabel='Coordinate 2') # remove the x axis label and the y axis label
plt.title('PKS/NRPS')#, fontsize = 150);
plt.legend(title = 'Average Distance (3)' , bbox_to_anchor=(1.01,1), loc='upper left', borderaxespad=0)
# plt.savefig(resultsDirectory+r'\Figure-7.pdf', dpi=300, bbox_inches="tight") # has to be before plt.show(), saves to wrking directory
plt.show()

#%% Figure 8: Requires Section 2 and 3.

# Load the data regarding the Distances for statistical comparison.
normDF = pd.read_excel(bactStatspath)
# Select only the relevant genera for this statistical comparison (Micromonospora and Micromonospora_E)
micromonosporaGenera = normDF['Distance:'].loc[normDF['Genera by BGC:'] == "Micromonospora"].to_numpy();
microEGenera = normDF['Distance:'].loc[normDF['Genera by BGC:'] == 'Micromonospora_E'].to_numpy();
# Run the wilcoxon rank sum test. (double-sided)
(stat1, stat2) = scipy.stats.ranksums(micromonosporaGenera, microEGenera);
# p-value annotation information
pairs = [('Micromonospora', 'Micromonospora_E')];
pvalues = [stat2];
formatted_pvalues = [f'p={pvalue:.2e}' for pvalue in pvalues]

# Create a dataframe containing only the plotted genera.
valuableGenera = normDF.loc[(normDF['Genera by BGC:'] == "Micromonospora") | (normDF['Genera by BGC:'] ==  "Micromonospora_E")];

# Plot the statistically annotated figure.
plt.figure(figsize=(100,100))
currentParam = "40"
sns.set_context('poster', font_scale=8, rc={'axes.linewidth':'10', 'grid.linewidth':'10',})#, rc={'xtick.labelsize':66});
sns.set_style("whitegrid")
ax = sns.boxplot(data = valuableGenera, x="Genera by BGC:", y="Distance:", orient='v', showmeans = True, 
                 fliersize=66, linewidth = 30, palette = 'deep',
                 meanprops={"marker": "D",
                            "markeredgecolor":"black",
                            "markerfacecolor":"firebrick",
                            "markersize":"50"})
# annotate
annotator = Annotator(ax, pairs, data=valuableGenera,x="Genera by BGC:",y="Distance:",orient='v',showmeans=True,
                      fliersize=66, linewidth = 30, palette = 'deep',
                      meanprops={"marker": "D",
                                 "markeredgecolor":"black",
                                 "markerfacecolor":"firebrick",
                                 "markersize":"50"})
annotator.configure(test="Mann-Whitney", text_format='star',loc='inside', line_width = 30)
annotator.apply_and_annotate()

ax.set_title("Distribution of BGC Distance to Closest GCF in BiG-SLiCE by Genera")
ax.set_xlabel(None)
ax.set_ylabel('Distance')
# plt.savefig(resultsDirectory+r'\Figure-8.pdf', dpi=300)
plt.show()
#%% Supplementary Figures: Requires Section 2, 3, 5, (6 OR 6.5).
    # MDS on all data, individual plots per class.
    # Like Figure 5, but not annotated, all of the dots are the same size.

# Make a dataframe using whichever values of data you want. This one is using chebyshev, can change to match your metrics of interest.
dfMDS = pd.DataFrame(dictMDS['chebyshev']['chebyshev_orig'], index = testT1)
dfMDS['Category'] = classList;
dfMDS['Genome'] = truncG;
# Get all 10 classes from this.
categoryList = dfMDS['Category'].unique().tolist();
# Plot each of the classes across the dataset.
counterSup1 = 1;
for i in categoryList:
    curData = dfMDS.loc[dfMDS['Category'] == i];
    plt.figure(figsize= (100,100));
    sns.set_context("poster", font_scale = 4)
    sns.set_style("whitegrid")
    splitPlot = sns.scatterplot(x=0,y=1, data=curData, s=10000, edgecolor="white")
    splitPlot.set(xlabel='Coordinate 1', ylabel='Coordinate 2')
    # Modify the title to change your distance metric.
    # plt.title('Multidimensional Scaling (Chebyshev) on ' + str(i) + ' $\it{Micromonosporaceae}$ BGCs.', fontsize = 150) # previous version
    # supp1_figTitle = r'''$\bf{Figure-S''' + str(counterSup1) + ''':}$
    
    
    # Multidimensional Scaling (Chebyshev) on ''' + str(i) + ''' $\it{Micromonosporaceae}$ BGCs.''';
    supp1_figTitle = r'''$\bf{Figure-S''' + str(counterSup1) + ''':}$
    
    
    ''';
    plt.title(supp1_figTitle, fontsize = 150)
    # replace characters not accepted by windows into ones that are (for saving in batches purposes)
    i = i.replace('/','-'); # replaces the "/" character with the "-" character
    i = i.replace(' ','-'); # replaces the " " space with the "-" character
    supp1_figCap ='''Coordinate 1
    
    
    Scatter plot of ''' + str(i) + ''' $\it{Micromonosporaceae}$ BGCs analyzed via multidimensional scaling using Chebyshev pairwise distance. 
    Each dot represents an individual BGC. Distance between BGCs is associated with GCFs in BiG-SLiCE they were most similar to. '''
    plt.xlabel(supp1_figCap) # basically, puts figure captions below the xlabel.
    # plt.savefig(resultsDirectory+r'\Supplementary-Figure-overallMDS-individualClass-' + i + '.pdf', dpi=300)
    plt.savefig(resultsDirectory+r'\S'+str(counterSup1)+'.pdf', dpi=300,bbox_inches="tight")
    counterSup1 = counterSup1+1;
    plt.show()
    plt.close("all")

#%% Supplementary Figures: Requires Section 2, 3, 5, (6, OR 6.5).
    # MDS on all data, individual annotated plots per class.
    # Like Figure 5, but annotated (not just PKS/NRPS on just 'chebyshev')

# Make a dataframe using whichever values of data you want. This one is using chebyshev, can change to match your metrics of interest.
dfMDS = pd.DataFrame(dictMDS['chebyshev']['chebyshev_orig'], index = testT1)
dfMDS['Category'] = classList;
dfMDS['Genome'] = truncG;
categoryList = dfMDS['Category'].unique().tolist();

# Create a dataframe to store the older information from storageDict (Distance & such).
storage2DF = pd.DataFrame();
# Remove low quality genomes & WMMC500:
    # Make a backup first.
    # IF ERRORS: REMOVE (this is for regeneration purposes)
storageDictBackup = copy.deepcopy(storageDict);
for i in ['a1363','b482','b486','c500']:
    storageDict.pop(i);

for i in list(storageDict.keys()):
    storage2DF = pd.concat([storage2DF,storageDict[i]])
# list comprehension (if/else statement). GOal: '> 900' if the BGC fell above 900 in BiGSLICE, otherwise '< 900'
# bigSLICEinfo = ['> 900' if storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] >= 900 else '< 900' for iter1 in testT1];
bigSLICEinfo = ['Did Not Cluster' if storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] >= 900 else 'Clustered' for iter1 in testT1];
# find the distance to the closest GCF
closestGCFDist = [storage2DF.loc[storage2DF['BGC'] == iter1].Distance.tolist()[0] for iter1 in superDict0['orig'].index.tolist()];
# find the distance to the X closest GCFs, then average them. Where X is set above as aveDist
aveDist = 3;
aveDistStorage = [sorted(superDict0['orig'].loc[iter2])[:aveDist] for iter2 in superDict0['orig'].index]; # list of lists
averageDist = [np.mean(entry1) for entry1 in aveDistStorage]; # average each list inside the list of lists
dfMDSTemp2 = dfMDS.copy(deep='true'); # create a backup.
# Append additional columns to the dataFrame.
dfMDSTemp2['BIG-SLICE'] = bigSLICEinfo;
dfMDSTemp2['Distance'] = closestGCFDist;
dfMDSTemp2['Average Distance'] = averageDist;
dfMDSTemp2['# of GCF Distances used:'] = aveDist;

dfMDS2 = dfMDSTemp2.copy(deep=True) # remap to a new name.

# Plot all the figures, each based on the BGC Class.
counterSup2 = 11;
for i in categoryList:
    curData = dfMDS2.loc[dfMDS2['Category'] == i]; # current cateogry (iterates through Saccharides, Hybrids, etc)
    plt.figure(figsize=(100,80));
    sns.set_context("poster", font_scale = 4)
    sns.set_style("whitegrid")
    g = sns.scatterplot(data=curData, x=0, y=1,hue='Average Distance' ,size = 'Average Distance',sizes = (2000,10000) , edgecolor='white'); # default hue
    g.set(xlabel='Coordinate 1', ylabel='Coordinate 2') # 
    supp2_figTitle = r'''$\bf{Figure-S''' + str(counterSup2) + ''':}$
    
    
    ''';
    # plt.title('Multidimensional Scaling (Chebyshev): ' + i, fontsize = 150)
    plt.title(supp2_figTitle, fontsize=150)
    plt.xlim(-2500, 3000) # hardcoded right now, because i want all the plots to look identical
    plt.ylim(-3500, 3500) # hardcoded, see above
    plt.legend(title = 'Average Distance (3)' , bbox_to_anchor=(1.01,1), loc='upper left', borderaxespad=0)
    # plt.tight_layout()
    # replace characters not accepted by windows into ones that are (for saving in batches purposes)
    i = i.replace('/','-'); # replaces the "/" character with the "-" character
    i = i.replace(' ','-'); # replaces the " " space with the "-" character
    supp2_figCap ='''Coordinate 1
    
    
    Scatter plot of ''' + str(i) + ''' $\it{Micromonosporaceae}$ BGCs analyzed via multidimensional scaling using Chebyshev pairwise distance. 
    Each dot represents an individual BGC. The hue and size of the dots were scaled based on the average distance of
    the individual BGC, across the entire dataset, to the closest 3 GCFs in BiG-SLiCE (T=900).'''
    plt.xlabel(supp2_figCap) # basically, puts figure captions below the xlabel.
    # plt.savefig(resultsDirectory+r'\Supplementary-Figure-overallMDS-individualClass-annotated-' + i + '.pdf', dpi=300)
    # plt.savefig(resultsDirectory+r'\S'+str(counterSup2)+'.pdf', dpi=300,bbox_inches="tight") # the pdf is too small to include the legend normally, so it just drags off the right side.
    counterSup2 = counterSup2+1;
    plt.show()
    plt.close('all') # memory issues if you don't close the figures.
    
#%% Supplementary Figure: Requires Sections 2, 3, 5, 6 or 6.5, 7
    # Data split by individual class, MDS performed on each slice, then visualized per class.
    # Like Figure 7, but not annotated (all the same color, same size). Also done for all metrics and all bgc classes used.

specificMets = ['euclidean','cosine', 'cityblock', 'chebyshev','l2','braycurtis','canberra','correlation'];
# l1, manhattan, and cityblock are the same.

# plot it
counterSup3 = 21;
for j in list(dictDFMDS.keys()):
    for i in specificMets:
        curData = dictDFMDS[j][j+"_"+i];
        plt.figure(figsize= (100,100));
        sns.set_context("poster", font_scale = 4)
        sns.set_style("whitegrid")
        g = sns.scatterplot(x=0, y=1, data=curData, s=10000,edgecolor="white");
        g.xaxis.grid(True);
        g.yaxis.grid(True)
        g.set(xlabel='Coordinate 1',ylabel='Coordinate 2')
        supp3_figTitle = r'''$\bf{Figure-S''' + str(counterSup3) + ''':}$
        
        
        ''';
        # plt.title('MDS (' + str(i) + ') on ' + str(j.split('_')[0]) + ' BGCs');
        plt.title(supp3_figTitle, fontsize = 150);
        newN=j.split('_')[0] # get everything before _orig
        newN=newN.replace('/','-'); # modify name for ease of saving in batches
        newN=newN.replace(' ','-'); # modify name for ease of saving in batches
        supp3_figCap ='''Coordinate 1
        
        
        Scatter plot of ''' + str(newN) + ''' $\it{Micromonosporaceae}$ BGCs analyzed via multidimensional scaling using ''' + str(i) + ''' pairwise distance. 
        Only ''' + str(newN) +  ''' BGCs were considered for this analysis. Each dot represents an individual BGC.
        Distance between ''' + str(newN) + ''' BGCs is associated with the GCFs in BiG_SLiCE they were most similar to (T=900).''';
        plt.xlabel(supp3_figCap) # basically, puts figure captions below the xlabel.
        # print(resultsDirectory+r'\Supplementary-Figure-individualMDS-individualClass-' + i + '-' +newN + '.pdf') # See how it's saved.
        # plt.savefig(resultsDirectory+r'\Supplementary-Figure-individualMDS-individualClass-' + i + '-' + newN + '.pdf', dpi=300) # Uncomment when ready to save.
        plt.savefig(resultsDirectory+r'\S'+str(counterSup3)+'.pdf', dpi=300, bbox_inches="tight")
        counterSup3 = counterSup3 + 1;
        plt.show()
        plt.close('all') # memory issues if you don't close the figures.
        
#%% Supplementary Figures: Requires Sections 2, 3, 5, 6 OR 6.5, 7.
    # Data split by individual class, MDS performed on each slice, then annotated with GCF information, and visualized per class.
    # Like Figure 7, but for all metrics & bgc classes.
    
specificMets = ['euclidean','cosine', 'cityblock', 'chebyshev','l2','braycurtis','canberra','correlation'];
# l1, manhattan, and cityblock are the same.

# plot it
counterSup4 = 101;
for j in list(dictDFMDS.keys()):
    for i in specificMets:
        curData = dictDFMDS[j][j+"_"+i];
        plt.figure(figsize= (100,100));
        sns.set_context("poster", font_scale = 4)
        sns.set_style("whitegrid")
        g = sns.scatterplot(x=0, y=1, data=curData, hue='Average Distance', size = 'Average Distance', sizes =(5000,15000),edgecolor="black");
        g.xaxis.grid(True);
        g.yaxis.grid(True)
        g.set(xlabel='Coordinate 1',ylabel='Coordinate 2')
        supp3_figTitle = r'''$\bf{Figure-S''' + str(counterSup4) + ''':}$
        
        
        ''';
        # plt.title('MDS (' + str(i) + ') on ' + str(j.split('_')[0]) + ' BGCs');
        plt.title(supp3_figTitle, fontsize=150);
        newN=j.split('_')[0] # get everything before _orig
        newN=newN.replace('/','-'); # modify name for ease of saving in batches
        newN=newN.replace(' ','-'); # modify name for ease of saving in batches
        plt.legend(title = 'Average Distance (3)' , bbox_to_anchor=(1.01,1), loc='upper left', borderaxespad=0)
        supp4_figCap ='''Coordinate 1
        
        
        Scatter plot of ''' + str(newN) + ''' $\it{Micromonosporaceae}$ BGCs analyzed via multidimensional scaling using ''' + str(i) + ''' pairwise distance. 
        Only ''' + str(newN) +  ''' BGCs were considered for this analysis. Each dot represents an individual BGC.
        The hue and size of the dots were scaled based on the average distance of the individual ''' + str(newN) +   ''' BGC to the closest 3 GCFs in BiG-SLiCE (T=900).'''
        plt.xlabel(supp4_figCap) # basically, puts figure captions below the xlabel.
        # print(resultsDirectory+r'\Supplementary-Figure-individualMDS-individualClass-annotated-' + i + '-' +newN + '.pdf') # See how it's saved.
        # plt.savefig(resultsDirectory+r'\Supplementary-Figure-individualMDS-individualClass-annotated-' + i + '-' + newN + '.pdf', dpi=300, bbox_inches="tight") # Uncomment when ready to save.
        plt.savefig(resultsDirectory+r'\S'+str(counterSup4)+'.pdf', dpi=300,bbox_inches="tight") # the pdf is too small to include the legend normally, so it just drags off the right side.
        counterSup4 = counterSup4+1;
        plt.show()
        plt.close('all'); # memory issues if you don't close the figures.
